@@ -16,11 +16,12 @@
 */
 #include "PajeSimulator.h"
 #include "PajeException.h"
-#ifndef Q_MOC_RUN
-#include <boost/foreach.hpp>
-#include <boost/tokenizer.hpp>
-#endif
 
+#define PAJENG_OUT_OF_CORE_NCALLS 100000
+
+int _pajeng_quiet = 0;
+int _pajeng_out_of_core = 0;
+int _pajeng_entity_hierarchy_dump = 0;
 int ignoreIncompleteLinks = 0;
 
 PajeSimulator::PajeSimulator ()
@@ -193,11 +194,8 @@ bool PajeSimulator::noImbricationLevel (void)
 
 void PajeSimulator::setLastKnownTime (PajeTraceEvent *event)
 {
-  std::string time = event->valueForField (PAJE_Time);
-  if (time.length()){
-    double evttime = atof(time.c_str());
-    lastKnownTime = evttime;
-  }
+  lastKnownTime = event->time();
+  return;
 }
 
 PajeColor *PajeSimulator::getColor (std::string color, PajeTraceEvent *event)
@@ -205,19 +203,58 @@ PajeColor *PajeSimulator::getColor (std::string color, PajeTraceEvent *event)
   PajeColor *ret = NULL;
   if (!color.empty()){
     std::vector<float> v;
-    boost::char_separator<char> sep(", ");
-    boost::tokenizer< boost::char_separator<char> > tokens(color, sep);
-    BOOST_FOREACH(std::string t, tokens) {
-      v.push_back (atof(t.c_str()));
+    std::stringstream line;
+    line << *event;
+    if (!color.c_str())
+      throw PajeDecodeException ("Could not understand color parameter in " +
+          line.str());
+    char *color_cstr = strdup(color.c_str());
+    char *saveptr;
+    if (!color_cstr)
+      throw PajeDecodeException ("Could not understand color parameter in " +
+          line.str());
+    char *tok = strtok_r(color_cstr, ", ", &saveptr);
+    while (tok) {
+      v.push_back(atof(tok));
+      tok = strtok_r(NULL, ", ", &saveptr);
     }
+    free(color_cstr);
     if (v.size()==3){
       ret = new PajeColor (v[0], v[1], v[2], 1);
     }else if (v.size()==4){
         ret = new PajeColor (v[0], v[1], v[2], v[3]);
     }else{
-      std::stringstream line;
-      line << *event;
-      throw PajeDecodeException ("Could not understand color parameter in "+line.str());
+      /* Assume hex color (coded like in HTML colors).
+	 Not a color if more than three componentes or not a hex number. */
+      int sz = color.size();
+      int comp = 0;
+      int cnt = 0;
+      float rgb[3] = {0, 0, 0};
+      bool errorFlag = false;
+      for (int i = 0; i < sz; i++) {
+        char c = color[i];
+        if (isdigit(c) || (c >= 'a' && c <= 'f') || (c >= 'A' && c <= 'F')){
+	  comp = comp * 16 + (isdigit(c) ? c - '0' : 10 +
+                              ((c >= 'a' && c <= 'f') ? c - 'a' : c - 'A' ));
+	  if (i % 2 == 1){
+	    if (cnt == 3) {
+	      errorFlag = true;
+	      break;
+	    }
+	    rgb[cnt++] = (float)comp / 256.0;
+	    comp = 0;
+	  }
+        }else{
+	  errorFlag = true;
+	  break;
+	}
+      }
+      if (cnt == 3 && errorFlag == false){
+        ret = new PajeColor (rgb[0], rgb[1], rgb[2], 1);
+      }else{
+	throw PajeDecodeException ("Could not understand color parameter in " +
+				   line.str());
+      }
     }
   }
   return ret;
@@ -245,7 +282,20 @@ void PajeSimulator::inputEntity (PajeObject *data)
       CALL_MEMBER_PAJE_SIMULATOR(*this,invocation[eventId])(event);
     }
   }else{
-    throw PajeSimulationException ("Unknow event id.");
+    throw PajeSimulationException ("Unknown event id.");
+  }
+
+  {
+    static unsigned long long ncalls = 0;
+    // If Out Of Core simulation, dump and release entities
+    if (_pajeng_out_of_core && ncalls >= PAJENG_OUT_OF_CORE_NCALLS){
+
+      // This function is top-down recursive
+      std::ostream stdoutStream (std::cout.rdbuf());
+      root->recursiveClearCompleteEntities(stdoutStream, !_pajeng_entity_hierarchy_dump);
+      ncalls = 0;
+    }
+    ncalls++;
   }
 }
 
@@ -264,6 +314,12 @@ void PajeSimulator::finishedReading (void)
   hierarchyChanged ();
   timeLimitsChanged ();
   setSelectionStartEndTime (startTime(), endTime());
+
+  // If Out Of Core simulation, dump and release entities
+  if (_pajeng_out_of_core){
+    std::ostream stdoutStream (std::cout.rdbuf());
+    root->recursiveClearCompleteEntities(stdoutStream, !_pajeng_entity_hierarchy_dump);
+  }
 }
 
 void PajeSimulator::pajeDefineContainerType (PajeTraceEvent *event)
@@ -285,7 +341,7 @@ void PajeSimulator::pajeDefineContainerType (PajeTraceEvent *event)
   if (!containerType){
     std::stringstream line;
     line << *event;
-    throw PajeTypeException ("Unknow container type '"+type+"' in "+line.str());
+    throw PajeTypeException ("Unknown container type '"+type+"' in "+line.str());
   }
 
   std::string identifier = !alias.empty() ? alias : name;
@@ -313,7 +369,7 @@ void PajeSimulator::pajeDefineLinkType (PajeTraceEvent *event)
   if (!containerType){
     std::stringstream line;
     line << *event;
-    throw PajeTypeException ("Unknow container type '"+type+"' in "+line.str());
+    throw PajeTypeException ("Unknown container type '"+type+"' in "+line.str());
   }
 
   //search for start container type
@@ -321,7 +377,7 @@ void PajeSimulator::pajeDefineLinkType (PajeTraceEvent *event)
   if (!startcontainertype){
     std::stringstream line;
     line << *event;
-    throw PajeTypeException ("Unknow start container type '"+starttype+"' for link definition in "+line.str());
+    throw PajeTypeException ("Unknown start container type '"+starttype+"' for link definition in "+line.str());
   }
 
   //search for end container type
@@ -329,7 +385,7 @@ void PajeSimulator::pajeDefineLinkType (PajeTraceEvent *event)
   if (!endcontainertype){
     std::stringstream line;
     line << *event;
-    throw PajeTypeException ("Unknow end container type '"+endtype+"' for link definition in "+line.str());
+    throw PajeTypeException ("Unknown end container type '"+endtype+"' for link definition in "+line.str());
   }
 
   //check if the new type already exists
@@ -356,7 +412,7 @@ void PajeSimulator::pajeDefineEventType (PajeTraceEvent *event)
   if (!containerType){
     std::stringstream line;
     line << *event;
-    throw PajeTypeException ("Unknow container type '"+type+"' in "+line.str());
+    throw PajeTypeException ("Unknown container type '"+type+"' in "+line.str());
   }
 
   std::string identifier = !alias.empty() ? alias : name;
@@ -382,7 +438,7 @@ void PajeSimulator::pajeDefineStateType (PajeTraceEvent *event)
   if (!containerType){
     std::stringstream line;
     line << *event;
-    throw PajeTypeException ("Unknow container type '"+type+"' in "+line.str());
+    throw PajeTypeException ("Unknown container type '"+type+"' in "+line.str());
   }
 
   std::string identifier = !alias.empty() ? alias : name;
@@ -409,7 +465,7 @@ void PajeSimulator::pajeDefineVariableType (PajeTraceEvent *event)
   if (!containerType){
     std::stringstream line;
     line << *event;
-    throw PajeTypeException ("Unknow container type '"+type+"' in "+line.str());
+    throw PajeTypeException ("Unknown container type '"+type+"' in "+line.str());
   }
 
   std::string identifier = !alias.empty() ? alias : name;
@@ -440,7 +496,7 @@ void PajeSimulator::pajeDefineEntityValue (PajeTraceEvent *event)
   if (!type){
     std::stringstream line;
     line << *event;
-    throw PajeTypeException ("Unknow type '"+typestr+"' in "+line.str());
+    throw PajeTypeException ("Unknown type '"+typestr+"' in "+line.str());
   }
 
   //check if the type accepts values
@@ -524,7 +580,7 @@ void PajeSimulator::pajeCreateContainer (PajeTraceEvent *traceEvent)
   }
 
   //verify if there is a container with the same name
-  std::string identifier = !alias.empty() ? alias : name;  
+  std::string identifier = !alias.empty() ? alias : name;
   PajeContainer *cont = contMap[identifier];
   if (cont){
     std::stringstream eventdesc;
