@@ -31,7 +31,17 @@ private:
   double t1, t2;
 
 public:
-  PajeUnity (bool flexReader, bool strictHeader, std::string tracefilename, double stopat, int ignoreIncompleteLinks, char *probabilistic, bool noImbrication);
+  PajeUnity (bool flexReader,
+	     bool strictHeader,
+	     std::string tracefilename,
+	     double stopat,
+	     int ignoreIncompleteLinks,
+	     char *probabilistic,
+	     bool noImbrication,
+	     bool userDefined,
+	     bool outOfCore,
+	     bool entityDump,
+	     bool quiet);
   ~PajeUnity ();
   double getTime ();
   void report ();
